import { NgModule, ɵɵdefineInjectable, ɵɵinject, Injectable, Inject } from '@angular/core';
import { StorageServiceModule, LOCAL_STORAGE } from 'ngx-webstorage-service';
import { VgCoreModule } from '@videogular/ngx-videogular/core';
import { VgControlsModule } from '@videogular/ngx-videogular/controls';
import { VgOverlayPlayModule } from '@videogular/ngx-videogular/overlay-play';
import { VgBufferingModule } from '@videogular/ngx-videogular/buffering';
import { of, Subject } from 'rxjs';
import { v4 } from 'uuid';
import Shepherd from 'shepherd.js';
import { NavigationEnd, Router } from '@angular/router';

class UserOnboardingLibModule {
}
UserOnboardingLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    StorageServiceModule,
                    VgCoreModule,
                    VgControlsModule,
                    VgOverlayPlayModule,
                    VgBufferingModule
                ],
                exports: []
            },] }
];

class SaveSCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const newTourState = this.parameters.state;
        this.storage.set(`${newTourState.sessionId}_${this.parameters.tourId}`, newTourState);
        return of(newTourState);
    }
}
class LoadSCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const currentState = this.storage.get(`${this.parameters.sessionId}_${this.parameters.tourId}`);
        return of(currentState);
    }
}
class DeleteSCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        this.storage.remove(`${this.parameters.sessionId}_${this.parameters.tourId}`);
    }
}

class SaveTCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const newTour = {
            tourId: this.parameters.tourId,
            tourSteps: this.parameters.tourSteps,
            styleSheet: this.parameters.styleSheet,
        };
        this.storage.set(this.parameters.tourId, newTour);
        return of(newTour);
    }
}
class LoadTCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        const existingTour = this.storage.get(this.parameters.tourId);
        return of(existingTour);
    }
}
class DeleteTCommand {
    constructor(storage) {
        this.storage = storage;
    }
    execute() {
        this.storage.remove(this.parameters.tourId);
    }
}

class TourStoreServiceService {
    constructor(storage) {
        this.storage = storage;
        this.commandMap = new Map();
        this.functionMap = new Map();
        this.defaultSSCommand = new SaveSCommand(this.storage);
        this.defaultSTCommand = new SaveTCommand(this.storage);
        this.defaultLSCommand = new LoadSCommand(this.storage);
        this.defaultLTCommand = new LoadTCommand(this.storage);
        this.defaultDSCommand = new DeleteSCommand(this.storage);
        this.defaultDTCommand = new DeleteTCommand(this.storage);
        this.sessionGenerator = () => v4();
        this.commandMap.set('SaveTourCommand', this.defaultSTCommand);
        this.commandMap.set('LoadTourCommand', this.defaultLTCommand);
        this.commandMap.set('SaveStateCommand', this.defaultSSCommand);
        this.commandMap.set('LoadStateCommand', this.defaultLSCommand);
        this.commandMap.set('DeleteStateCommand', this.defaultDSCommand);
        this.commandMap.set('DeleteTourCommand', this.defaultDTCommand);
    }
    registerSaveTourCommand(cmd) {
        this.commandMap.set('SaveTourCommand', cmd);
    }
    registerLoadTourCommand(cmd) {
        this.commandMap.set('LoadTourCommand', cmd);
    }
    registerSaveStateCommand(cmd) {
        this.commandMap.set('SaveStateCommand', cmd);
    }
    registerLoadStateCommand(cmd) {
        this.commandMap.set('LoadStateCommand', cmd);
    }
    registerDeleteStateCommand(cmd) {
        this.commandMap.set('DeleteStateCommand', cmd);
    }
    registerDeleteTourCommand(cmd) {
        this.commandMap.set('DeleteTourCommand', cmd);
    }
    saveTour(parameters) {
        const command = this.commandMap.get('SaveTourCommand');
        command.parameters = parameters;
        return command.execute();
    }
    loadTour(parameters) {
        const command = this.commandMap.get('LoadTourCommand');
        command.parameters = parameters;
        return command.execute();
    }
    deleteTour(parameters) {
        const command = this.commandMap.get('DeleteTourCommand');
        command.parameters = parameters;
        return command.execute();
    }
    saveState(parameters) {
        const command = this.commandMap.get('SaveStateCommand');
        command.parameters = parameters;
        return command.execute();
    }
    loadState(parameters) {
        const command = this.commandMap.get('LoadStateCommand');
        command.parameters = parameters;
        return command.execute();
    }
    deleteState(parameters) {
        const command = this.commandMap.get('DeleteStateCommand');
        command.parameters = parameters;
        return command.execute();
    }
    registerFnRef(key, fn) {
        this.functionMap.set(key, fn);
    }
    getFnByKey(key) {
        return this.functionMap.get(key);
    }
    generateSessionId() {
        this.sessionId = this.sessionGenerator();
        this.storage.set('TOUR_SESSION_ID', this.sessionId);
    }
    getSessionId() {
        return this.storage.get('TOUR_SESSION_ID');
    }
    setSessionIdGenerator(fn) {
        this.sessionGenerator = fn;
    }
}
TourStoreServiceService.ɵprov = ɵɵdefineInjectable({ factory: function TourStoreServiceService_Factory() { return new TourStoreServiceService(ɵɵinject(LOCAL_STORAGE)); }, token: TourStoreServiceService, providedIn: "root" });
TourStoreServiceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
TourStoreServiceService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [LOCAL_STORAGE,] }] }
];

const prefix = `<vg-player><vg-overlay-play></vg-overlay-play><vg-buffering></vg-buffering>
<vg-scrub-bar><vg-scrub-bar-current-time></vg-scrub-bar-current-time><vg-scrub-bar-buffering-time></vg-scrub-bar-buffering-time></vg-scrub-bar>
<vg-controls><vg-play-pause></vg-play-pause> <vg-playback-button></vg-playback-button><vg-time-display vgProperty="current" vgFormat="mm:ss">
</vg-time-display><vg-scrub-bar style="pointer-events: none;"></vg-scrub-bar><vg-time-display vgProperty="left" vgFormat="mm:ss"></vg-time-display>
<vg-time-display vgProperty="total" vgFormat="mm:ss"></vg-time-display><vg-track-selector></vg-track-selector><vg-mute ></vg-mute><vg-volume></vg-volume>
<vg-fullscreen></vg-fullscreen></vg-controls>
<video width="100%" controls disablepictureinpicture controlslist="nodownload" [vgMedia]="$any(media)" #media id= "singleVideo" preload="auto" controls><source src = "`;
const suffix = '"type = video/webm type="video/mp4"> </video></vg-player>';

var Status;
(function (Status) {
    Status["InProgress"] = "inProgress";
    Status["Complete"] = "complete";
})(Status || (Status = {}));

const INTERVAL = 100;
const DEFAULT_MAX_WAIT_TIME = 3000;

class TourServiceService {
    constructor(tourStoreService, router) {
        this.tourStoreService = tourStoreService;
        this.router = router;
        this.tourComplete = new Subject();
        this.tourComplete$ = this.tourComplete.asObservable();
        this.tourStepChange = new Subject();
        this.tourStepChange$ = this.tourStepChange.asObservable();
        this.interval = INTERVAL;
        this._maxWaitTime = DEFAULT_MAX_WAIT_TIME;
        this.tourFailed = new Subject();
        this.tourFailed$ = this.tourFailed.asObservable();
    }
    set maxWaitTime(maxTime) {
        if (maxTime > 0) {
            this._maxWaitTime = maxTime;
        }
    }
    get maxWaitTime() {
        return this._maxWaitTime;
    }
    addRemovedSteps(removedSteps) {
        const count = removedSteps.length;
        for (let i = 0; i < count; i++) {
            this.tour.steps.splice(0, 0, this.tour.steps.pop());
        }
    }
    actionAssignment(e, b, wrapperNormalNext, wrapperNormalPrev, wrapperNext, wrapperPrev, func, tourId, props) {
        if (b.key === 'prevAction') {
            b.action =
                e.prevRoute === e.currentRoute ? wrapperNormalPrev : wrapperPrev;
        }
        else if (b.key === 'nextAction') {
            b.action =
                e.nextRoute === e.currentRoute ? wrapperNormalNext : wrapperNext;
        }
        else {
            b.action = func.bind({ tour: this.tour, tourId, props });
        }
    }
    navigationCheckNext(event, e, tourInstance, currentStepId, previousStepId) {
        if (event instanceof NavigationEnd && event.url === e.currentRoute) {
            this.waitForElement(e, tourInstance.tourId)
                .then(() => {
                this.tour.next();
                this.tourStepChange.next({
                    tourId: tourInstance.tourId,
                    currentStepId,
                    previousStepId,
                    moveForward: true,
                });
                this.pauseAllVideos();
            })
                .catch(err => {
                this.tourFailed.next(err);
            });
        }
    }
    navigationCheckBack(event, e, tourInstance, currentStepId, previousStepId) {
        if (event instanceof NavigationEnd && event.url === e.currentRoute) {
            this.waitForElement(e, tourInstance.tourId)
                .then(() => {
                this.tour.back();
                this.tourStepChange.next({
                    tourId: tourInstance.tourId,
                    currentStepId,
                    previousStepId,
                    moveForward: false,
                });
                this.pauseAllVideos();
            })
                .catch(err => {
                this.tourFailed.next(err);
            });
        }
    }
    waitForElement(tourStep, tourId) {
        const startTime = new Date().getTime();
        return new Promise((resolve, reject) => {
            const timer = setInterval(() => {
                const now = new Date().getTime();
                const element = this.checkElement(tourStep.attachTo);
                if (element) {
                    clearInterval(timer);
                    resolve(element);
                }
                else if (now - startTime >= this._maxWaitTime) {
                    clearInterval(timer);
                    reject({ tourId, message: `Error in loading tour` });
                }
                else {
                    // do nothing
                }
            }, this.interval);
        });
    }
    triggerTour(tourInstance, props) {
        let removedSteps;
        const sessionId = this.tourStoreService.getSessionId();
        if (!sessionId) {
            this.tourStoreService.generateSessionId();
        }
        this.tourStoreService
            .loadState({ tourId: tourInstance.tourId, sessionId })
            .subscribe(tourState => {
            if (tourState && Object.keys(tourState).length) {
                removedSteps = this.getRemovedSteps(tourInstance.tourSteps, tourState);
                tourInstance.tourSteps = this.getTourSteps(tourInstance.tourSteps, tourState);
            }
            else {
                tourState = {
                    sessionId: this.tourStoreService.getSessionId(),
                    step: tourInstance.tourSteps[0].id,
                    props,
                    status: Status.InProgress,
                };
                this.tourStoreService
                    .saveState({
                    tourId: tourInstance.tourId,
                    state: tourState,
                })
                    .subscribe();
            }
            tourInstance.tourSteps.forEach(e => {
                e.buttons.forEach(b => {
                    const key = b.key;
                    const func = this.tourStoreService.getFnByKey(key);
                    const wrapperNext = () => {
                        this.tourStoreService
                            .saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.nextStepId,
                                props: tourState.props,
                                status: this.getStatus(e, tourInstance.tourSteps),
                            },
                        })
                            .subscribe();
                        this.router.navigate([e.nextRoute]);
                        this.router.events.subscribe((event) => {
                            const nextStep = tourInstance.tourSteps.filter(ts => ts.id === e.nextStepId)[0];
                            this.navigationCheckNext(event, nextStep, tourInstance, e.nextStepId, e.id);
                        });
                    };
                    const wrapperPrev = () => {
                        this.tourStoreService
                            .saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.prevStepId,
                                props: tourState.props,
                                status: Status.InProgress,
                            },
                        })
                            .subscribe();
                        this.router.navigate([e.prevRoute]);
                        this.router.events.subscribe((event) => {
                            const prevStep = tourInstance.tourSteps.filter(ts => ts.id === e.prevStepId)[0];
                            this.navigationCheckBack(event, prevStep, tourInstance, e.prevStepId, e.id);
                        });
                    };
                    const wrapperNormalNext = () => {
                        this.tourStoreService
                            .saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.nextStepId,
                                props: tourState.props,
                                status: this.getStatus(e, tourInstance.tourSteps),
                            },
                        })
                            .subscribe();
                        this.tour.next();
                        this.tourStepChange.next({
                            tourId: tourInstance.tourId,
                            currentStepId: e.nextStepId,
                            previousStepId: e.id,
                            moveForward: true,
                        });
                        this.pauseAllVideos();
                    };
                    const wrapperNormalPrev = () => {
                        this.tourStoreService
                            .saveState({
                            tourId: tourInstance.tourId,
                            state: {
                                sessionId: this.tourStoreService.getSessionId(),
                                step: e.prevStepId,
                                props: tourState.props,
                                status: Status.InProgress,
                            },
                        })
                            .subscribe();
                        this.tour.back();
                        this.tourStepChange.next({
                            tourId: tourInstance.tourId,
                            currentStepId: e.prevStepId,
                            previousStepId: e.id,
                            moveForward: true,
                        });
                        this.pauseAllVideos();
                    };
                    this.actionAssignment(e, b, wrapperNormalNext, wrapperNormalPrev, wrapperNext, wrapperPrev, func, tourInstance.tourId, tourState.props);
                });
            });
            const originalSteps = Object.assign([], tourInstance.tourSteps);
            this.tour.addSteps(tourInstance.tourSteps);
            this.waitForElement(originalSteps[0], tourInstance.tourId).then(element => {
                element.scrollIntoView(true);
                this.tour.start();
                if (removedSteps !== undefined) {
                    removedSteps.forEach(er => {
                        er.buttons.forEach(br => {
                            const k = br.key;
                            const funcRemoved = this.tourStoreService.getFnByKey(k);
                            const wrapperNextRemoved = () => {
                                this.tourStoreService
                                    .saveState({
                                    tourId: tourInstance.tourId,
                                    state: {
                                        sessionId: this.tourStoreService.getSessionId(),
                                        step: er.nextStepId,
                                        props: tourState.props,
                                        status: this.getStatus(er, tourInstance.tourSteps),
                                    },
                                })
                                    .subscribe();
                                this.router.navigate([er.nextRoute]);
                                this.router.events.subscribe((event) => {
                                    this.navigationCheckNext(event, er, tourInstance, er.nextStepId, er.id);
                                });
                            };
                            const wrapperPrevRemoved = () => {
                                this.tourStoreService
                                    .saveState({
                                    tourId: tourInstance.tourId,
                                    state: {
                                        sessionId: this.tourStoreService.getSessionId(),
                                        step: er.prevStepId,
                                        props: tourState.props,
                                        status: Status.InProgress,
                                    },
                                })
                                    .subscribe();
                                this.router.navigate([er.prevRoute]);
                                this.router.events.subscribe((event) => {
                                    this.navigationCheckBack(event, er, tourInstance, er.prevStepId, er.id);
                                });
                            };
                            const wrapperNormalNextRemoved = () => {
                                this.tourStoreService
                                    .saveState({
                                    tourId: tourInstance.tourId,
                                    state: {
                                        sessionId: this.tourStoreService.getSessionId(),
                                        step: er.nextStepId,
                                        props: tourState.props,
                                        status: this.getStatus(er, tourInstance.tourSteps),
                                    },
                                })
                                    .subscribe();
                                this.tour.next();
                                this.tourStepChange.next({
                                    tourId: tourInstance.tourId,
                                    currentStepId: er.nextStepId,
                                    previousStepId: er.id,
                                    moveForward: true,
                                });
                                this.pauseAllVideos();
                            };
                            const wrapperNormalPrevRemoved = () => {
                                this.tourStoreService
                                    .saveState({
                                    tourId: tourInstance.tourId,
                                    state: {
                                        sessionId: this.tourStoreService.getSessionId(),
                                        step: er.prevStepId,
                                        props: tourState.props,
                                        status: Status.InProgress,
                                    },
                                })
                                    .subscribe();
                                this.tour.back();
                                this.tourStepChange.next({
                                    tourId: tourInstance.tourId,
                                    currentStepId: er.nextStepId,
                                    previousStepId: er.id,
                                    moveForward: false,
                                });
                                this.pauseAllVideos();
                            };
                            this.actionAssignment(er, br, wrapperNormalNextRemoved, wrapperNormalPrevRemoved, wrapperNextRemoved, wrapperPrevRemoved, funcRemoved, tourInstance.tourId, tourState.props);
                        });
                    });
                    this.tour.addSteps(removedSteps);
                    this.addRemovedSteps(removedSteps);
                }
            }, err => {
                this.tourFailed.next(err);
            });
        });
    }
    run(tourId, params, props, filterFn) {
        this.tourStoreService
            .loadTour({
            tourId,
            sessionId: this.tourStoreService.getSessionId(),
        })
            .subscribe(tourInstance => {
            if (tourInstance && tourInstance.tourSteps.length > 0) {
                if (params) {
                    let steps = JSON.stringify(tourInstance.tourSteps);
                    Object.keys(params).forEach(key => {
                        steps = steps.replace(new RegExp(`\\{\\{${key}\\}\\}`), params[key]);
                    });
                    tourInstance.tourSteps = JSON.parse(steps);
                }
                this.tour = new Shepherd.Tour({
                    useModalOverlay: true,
                    defaultStepOptions: {
                        cancelIcon: {
                            enabled: true,
                        },
                        classes: 'class-1 class-2',
                        scrollTo: { behavior: 'smooth', block: 'center' },
                    },
                });
                this.tour.on('complete', (event) => {
                    event.tourId = tourInstance.tourId;
                    this.tourComplete.next(event);
                });
                if (filterFn) {
                    tourInstance.tourSteps = filterFn(tourInstance.tourSteps);
                }
                tourInstance.tourSteps[0].attachTo.scrollTo = false;
                this.triggerTour(tourInstance, props);
            }
            else {
                // do nothing
            }
        });
    }
    checkElement(attachTo) {
        switch (attachTo.type) {
            case 'string':
                return document.querySelector(attachTo.element);
            case 'element':
                return attachTo.element;
            case 'function':
                const fn = this.tourStoreService.getFnByKey(attachTo.element);
                return fn();
            default:
                return false;
        }
    }
    getRemovedSteps(tourSteps, tourState) {
        let f = true;
        return tourSteps.filter(e => {
            if (e.id === tourState.step || !f) {
                f = false;
            }
            return f;
        });
    }
    getTourSteps(tourSteps, tourState) {
        let flag = false;
        return tourSteps.filter(e => {
            if (e.id === tourState.step || flag) {
                flag = true;
            }
            return flag;
        });
    }
    getStatus(currentTourStep, tourSteps) {
        const lastId = tourSteps[tourSteps.length - 1].id;
        if (currentTourStep.id === lastId) {
            return Status.Complete;
        }
        else {
            return Status.InProgress;
        }
    }
    pauseAllVideos() {
        document.querySelectorAll('video').forEach(vid => vid.pause());
    }
}
TourServiceService.ɵprov = ɵɵdefineInjectable({ factory: function TourServiceService_Factory() { return new TourServiceService(ɵɵinject(TourStoreServiceService), ɵɵinject(Router)); }, token: TourServiceService, providedIn: "root" });
TourServiceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root',
            },] }
];
TourServiceService.ctorParameters = () => [
    { type: TourStoreServiceService },
    { type: Router }
];

/*
 * Public API Surface of user-onboarding-lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Status, TourServiceService, TourStoreServiceService, UserOnboardingLibModule, prefix, suffix };
//# sourceMappingURL=sourceloop-user-onboarding-client.js.map
